/*
 *  Copyright (c) 1992-2012 Matisse Software Inc. All Rights Reserved.
 *
 *  This file (in both binary and source code form) is subject to the
 *  Supplemental License Terms governing use, modification and redistribution
 *  of Sample Code accompanying the Matisse(R) software.
 *
 */
 
// the default Matisse C++ header
#include "matisseCXX.h"

// the namespaces that contain the Matisse classes
using namespace matisse;
using namespace matisse::reflect;

// A more complicated connection example which can be controlled by
// environment variables to specify additional access parameters, such
// as read-only access, user privleges, etc.  NOTE:  Security must be
// enabled for a database before users may be specified; for details,
// see section 5 of the Matisse Server Administration Guide.
class AdvancedConnect
{
private:
  MtDatabase db;

public:
  // constructor creates the MtDatabase
  AdvancedConnect(const std::string& host, const std::string& dbname)
    : db(host,dbname)
  {
  }

  void run()
  {
    if (getenv("MT_MEM_TRANS") != NULL)
      db.setOption(::MT_MEMORY_TRANSPORT, 1);

    // possible values are 0=RW, 1=RO, 2=DD (RW + Schema Modification)
    if (getenv("MT_DATA_ACCESS") != NULL)
      {
	int da_opt = atoi(getenv("MT_DATA_ACCESS"));
	db.setOption(::MT_DATA_ACCESS_MODE, da_opt);
      }

    if (getenv("dbuser") != NULL)
      {
	std::string user = getenv("dbuser") ? getenv("dbuser") : "";
	std::string passwd = getenv("dbpasswd") ? getenv("dbpasswd") : "";
	db.open(user, passwd);
      }
    else
      {
	db.open();
      }

    start(isReadOnly());
    printState();

    // do other work here ...

    end();
    db.close();
  }

  void start(bool readonly)
  {
    if (readonly)
      db.startVersionAccess();
    else
      db.startTransaction();
  }

  void end(void)
  {
    if (db.isVersionAccessInProgress())
      db.endVersionAccess();
    else if (db.isTransactionInProgress())
      db.commit();
    else
      std::cerr << "No transaction/version access in progress"
		<< std::endl;
  }

  bool isMemoryTransportOn()
  {
    return (db.getOption(::MT_TRANSPORT_TYPE) == ::MT_MEM_TRANSPORT);
  }

  bool isReadOnly()
  {
    return (db.getOption(::MT_DATA_ACCESS_MODE) == 1);
  }

  void printState()
  {
    if (!db.isConnectionOpen())
      {
	dbmsg("not connected");
      }
    else
      {
	if (db.isTransactionInProgress())
	  dbmsg("read-write transaction underway");
	else if (db.isVersionAccessInProgress())
	  dbmsg("read-only version access underway");
	else
	  dbmsg("no transaction underway");
      }
    std::string msg = "MEMORY_TRANSPORT is ";
    msg += (isMemoryTransportOn() ? "on" : "off");
    dbmsg(msg);
    msg = "Access is ";
    msg += (isReadOnly() ? "readonly" : "readwrite");
    dbmsg(msg);
  }

  void dbmsg(const std::string& msg)
  {
    std::cout << db << ": " << msg << std::endl;
  }
};

int main(int ac, char **av)
{
    if (ac < 3)
    {
        std::cerr << "Usage: " << av[0] << " <host> <db>" << std::endl;
        return -1;
    }

    try {
        AdvancedConnect conn(av[1], av[2]);
        conn.run();
    }
    catch (MtException& mte)
    {
        std::cerr << mte << std::endl;
    }
    catch (...)
    {
        std::cerr << "Unknown exception" << std::endl;
    }

}
