/*
 *  Copyright (c) 1992-2012 Matisse Software Inc. All Rights Reserved.
 *
 *  This file (in both binary and source code form) is subject to the
 *  Supplemental License Terms governing use, modification and redistribution
 *  of Sample Code accompanying the Matisse(R) software.
 *
 */

#include "matisseCXX.h"

#include "company.h"

#include <iostream>
#include <sstream>
#include <iomanip>

using namespace std;

// namespaces containing Matisse classes
using namespace matisse;

// namespaces containing the Schema classes
using namespace examples::cxx_examples::chaps_10_11;


// Sample application demonstrates adding relationship successors one at
// a time in separate transactions in order to handle any exceptions
// individually.  The advantage of this approach is that exceptions will
// will roll back only the transactions involving invalid data and other
// transactions will commit successfully.  (If all the updates were
// attempted in the same transaction, one exception would result in all
// being rolled back.)
//
// Compare newman2.cpp to see the differences when using SQL to
// accomplish the same results
//
// *** Usage ***
//
// NOTE:  Before running newman1, run load to generate sample data for
// the Employee class.
//
// newman1  <host> <db> <asstFirstName> <asstLastName> <count> <id>
// will create <count> managers starting with ID number <id>, each with
// the assistant Employee whose first and last name are specified.
//
// Use the <id> option to generate exceptions deliberately.  For example:
//
// newman1 <host> <database> JOHN DOE000001 1 2000
// newman1 <host> <database> JOHN DOE000098 2 2000
//
// Since the first command creates a manager with the ID 2000, the
// second command's attempt to create another with the same ID will
// fail and be rolled back.

int main(int ac, char **av)
{
  int actualCreateCount = 0, actualTransCount = 0, actualAbortCount = 0;
  int newManagerCount = 50;   // default

  if (ac < 5)
  {
    std::cerr << "Usage: " << av[0]
              << " <host> <db> <asstFirstName> <asstLastName> [<count> <id>]"
              << std::endl;
    return -1;
  }

  try
  {
    MtDatabase db(av[1], av[2]);
    int useId = 1000;        // default starting id for manager

    std::string asstFirstName = av[3];
    std::string asstLastName = av[4];

    if (ac > 5)
      newManagerCount = ::atoi(av[5]);
    if (ac > 6)
      useId = ::atoi(av[6]);

    db.open();

    std::cout << "Creating " << newManagerCount
              << " managers with an assistant of "
              << asstFirstName << " " << asstLastName << std::endl;
    for (int i = 0; i < newManagerCount; i++)
    {
      try
      {
        actualTransCount++;
        db.startTransaction();

        // find the employee by name lookup; in this simple
        // example, this lookup should be moved outside the
        // loop, since the search criteria do not change, and
        // left here for example only as we use the logic of
        // finding Employee to commit/rollback transaction.
        Employee *e =
          dynamic_cast<Employee*>(Person::lookupPersonName(db, asstLastName, asstFirstName));

        if (e != NULL)
        {
          // create a new manager
          Manager &m = Manager::create(db);
          std::ostringstream os1;
          os1 << "JACK";

          std::ostringstream os2;
          os2 << "MORRISON" << std::setw(6) << setfill('0') << useId;

          // manager attributes
          m.setFirstName(os1.str());
          m.setLastName(os2.str());
          m.setId(useId);
          // set the assistant
          m.setAssistant(*e);

          db.commit();
          actualCreateCount++;
        }
        else
        {
          std::cerr << "could not find " << asstFirstName << " "
                    << asstLastName << std::endl;
          if (db.isTransactionInProgress())
          {
            db.rollback();
          }
          actualAbortCount++;
        }
      }
      // an exception during create will rollback transaction
      catch (MtException& mte)
      {
        if (db.isTransactionInProgress())
        {
          db.rollback();
        }
        actualAbortCount++;
        std::cerr << mte << std::endl;
        std::cerr << "At transaction #: " << i
                  << ", id: " << useId << std::endl;
      }
      useId++;            // update for the next employee id
    }
    db.close();

  }
  catch (MtException& mte)
  {
    std::cerr << mte << std::endl;
  }
  catch (std::exception& e)
  {
    std::cerr << "Standard exception: " << e.what() << std::endl;
  }
  catch (...)
  {
    std::cerr << "Unknown exception" << std::endl;
  }

  std::cout << "Summary: " << actualCreateCount
            << " managers created in " << actualTransCount
            << " transactions, with " << actualAbortCount << " rollbacks"
            << std::endl;

  return 0;
}
